package be.iminds.ilabt.jfed.experiment.setup.config;

import com.fasterxml.jackson.annotation.*;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.Collections;
import java.util.List;

@JsonInclude(JsonInclude.Include.NON_DEFAULT)
@JsonIgnoreProperties(value={"@context"})
public class GenerateSimpleRequestRSpec extends RequestRSpec {
    private final int nodeCount;
    @Nonnull
    private final String nodeSliverType;
    private final boolean nodeExclusive;
    @Nonnull private final List<String> nodeComponentUrn;
    @Nonnull private final List<String> nodeAnsibleGroup;

    public GenerateSimpleRequestRSpec() {
        super(RequestRSpec.RequestRSpecSource.GENERATE_SIMPLE, false, false);
        this.nodeCount = 1;
        this.nodeSliverType = "raw-pc";
        this.nodeExclusive = true;
        this.nodeComponentUrn = Collections.emptyList();
        this.nodeAnsibleGroup = Collections.emptyList();
    }

    @JsonCreator
    public GenerateSimpleRequestRSpec(@Nonnull @JsonProperty("source") RequestRSpecSource source,
                                      @Nullable @JsonProperty("nodeCount") Integer nodeCount,
                                      @Nullable @JsonProperty("nodeSliverType") String nodeSliverType,
                                      @Nullable @JsonProperty("nodeExclusive") Boolean nodeExclusive,
                                      @Nullable @JsonProperty("nodeComponentUrn") List<String> nodeComponentUrn,
                                      @Nullable @JsonProperty("nodeAnsibleGroup") List<String> nodeAnsibleGroup,
                                      @Nullable @JsonProperty("autoAssignSpecificNodesUsingListResources") Boolean autoAssignSpecificNodesUsingListResources,
                                      @Nullable @JsonProperty("useNitosLease") Boolean useNitosLease) {
        super(source, autoAssignSpecificNodesUsingListResources, useNitosLease);
        assert source == RequestRSpec.RequestRSpecSource.GENERATE_SIMPLE;
        assert this.getSource() == RequestRSpec.RequestRSpecSource.GENERATE_SIMPLE;

        this.nodeCount = nodeCount == null ? new GenerateSimpleRequestRSpec().getNodeCount() : nodeCount;
        this.nodeSliverType = nodeSliverType == null ? new GenerateSimpleRequestRSpec().getNodeSliverType() : nodeSliverType;
        this.nodeExclusive = nodeExclusive == null ? new GenerateSimpleRequestRSpec().getNodeExclusive() : nodeExclusive;
        this.nodeComponentUrn = nodeComponentUrn == null ? new GenerateSimpleRequestRSpec().getNodeComponentUrn() : nodeComponentUrn;
        this.nodeAnsibleGroup = nodeAnsibleGroup == null ? new GenerateSimpleRequestRSpec().getNodeAnsibleGroup() : nodeAnsibleGroup;
    }

    @JsonIgnore
    public boolean isValid() {
        if (nodeCount <= 0)
            throw new IllegalArgumentException("Config Error: rspec.nodeCount == "+nodeCount+" makes no sense if rspec.source == "+source);
        return true;
    }

    @JsonProperty
    public int getNodeCount() {
        return nodeCount;
    }

    @JsonProperty
    @Nonnull
    public String getNodeSliverType() {
        return nodeSliverType;
    }

    @JsonProperty
    public boolean getNodeExclusive() {
        return nodeExclusive;
    }

    @JsonProperty
    @Nonnull
    public List<String> getNodeComponentUrn() {
        return nodeComponentUrn;
    }

    @JsonProperty
    @Nonnull
    public List<String> getNodeAnsibleGroup() {
        return nodeAnsibleGroup;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof GenerateSimpleRequestRSpec)) return false;
        if (!super.equals(o)) return false;

        GenerateSimpleRequestRSpec that = (GenerateSimpleRequestRSpec) o;

        if (nodeCount != that.nodeCount) return false;
        if (nodeExclusive != that.nodeExclusive) return false;
        if (!nodeSliverType.equals(that.nodeSliverType)) return false;
        if (!nodeAnsibleGroup.equals(that.nodeAnsibleGroup)) return false;
        return nodeComponentUrn.equals(that.nodeComponentUrn);
    }

    @Override
    public int hashCode() {
        int result = super.hashCode();
        result = 31 * result + nodeCount;
        result = 31 * result + nodeSliverType.hashCode();
        result = 31 * result + (nodeExclusive ? 1 : 0);
        result = 31 * result + nodeComponentUrn.hashCode();
        result = 31 * result + nodeAnsibleGroup.hashCode();
        return result;
    }
}
