package be.iminds.ilabt.jfed.highlevel.jobs.report;

import be.iminds.ilabt.jfed.call_log_output.LogOutput;
import be.iminds.ilabt.jfed.experiment.tasks.ExperimentTaskStatus;
import be.iminds.ilabt.jfed.highlevel.controller.TaskExecution;
import be.iminds.ilabt.jfed.highlevel.jobs.State;
import javafx.beans.property.ObjectProperty;
import javafx.beans.property.SimpleObjectProperty;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;

import javax.annotation.Nonnull;
import java.time.Instant;

/**
 * User: twalcari
 * Date: 12/9/2015
 * Time: 16:03
 */
public class JobStateReport {

    @Nonnull private final String stateName;
    @Nonnull private final Instant startTime;
    private final ObjectProperty<Instant> endTime = new SimpleObjectProperty<>();

    private final ObservableList<TaskExecution<?>> taskExecutions = FXCollections.observableArrayList();
    private final ObjectProperty<ExperimentTaskStatus> finalStatus = new SimpleObjectProperty<>();

    private final ObservableList<LogOutput.LogEntry> logLines = FXCollections.synchronizedObservableList(FXCollections.observableArrayList());


    public JobStateReport(@Nonnull State state) {
        this.stateName = state.getName();

        this.startTime = Instant.now();
    }

    public void registerEnd(@Nonnull ExperimentTaskStatus finalStatus) {
        this.endTime.set(Instant.now());
        this.finalStatus.set(finalStatus);

    }

    public void addTaskExecution(@Nonnull TaskExecution<?> taskExecution) {
        taskExecutions.add(taskExecution);
    }

    public void addLogLine(@Nonnull LogOutput.LogEntry logEntry) {
        this.logLines.add(logEntry);
    }

    @Nonnull public String getStateName() {
        return stateName;
    }

    @Nonnull public Instant getStartTime() {
        return startTime;
    }

    public Instant getEndTime() {
        return endTime.get();
    }

    public ObjectProperty<Instant> endTimeProperty() {
        return endTime;
    }

    public ObservableList<TaskExecution<?>> getTaskExecutions() {
        return taskExecutions;
    }

    public ExperimentTaskStatus getFinalStatus() {
        return finalStatus.get();
    }

    public ObjectProperty<ExperimentTaskStatus> finalStatusProperty() {
        return finalStatus;
    }

    public ObservableList<LogOutput.LogEntry> getLogLines() {
        return logLines;
    }
}
